      program runatom
c
c----------------------------------------------------------------------------
c
c     program for obtaining self-consistent solutions of the
c     kohn-sham equations for atoms and generating ultra soft
c     separable pseudopotentials
c
c     written by dominic king-smith from programs due to david vanderbilt
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
      parameter( idim1 = 1000      , idim2 = 26         )
      parameter( idim3 = 7         , idim4 = 4          )
      parameter( idim5 = 3         , idim6 = 2*idim5-1  )
      parameter( idim7 = 4         , idim8 = 10         )
c
c     idim1  .ge.  no. of points in the radial mesh
c     idim2  .ge.  no. of shells in the atom
c     idim3  .ge.  no. of beta functions in vanderbilt potential
c     idim4  .ge.  no. of valence states for pseudization
c     idim5  .ge.  lmax + 1, lmax is maximum l value of potential
c     idim7  .ge.  no. of reference states for each angular momentum value
c     idim8  .ge.  no. of terms in taylor expansion of pseudized q_i,j
c
c.....logarithmic radial mesh information
      dimension r(idim1),rab(idim1),sqr(idim1)
c.....potentials for the all electron calculation
      dimension ruae(idim1),runuc(idim1)
c.....hartree, exchange and correlation potentials
      dimension ruhar(idim1),reexch(idim1),ruexch(idim1)
c.....charge densities
      dimension rscore(idim1),rsvale(idim1),rsatom(idim1),rspsco(idim1)
c.....wavefunctions
      dimension snl(idim1,idim2),snlo(idim1),yy(idim1,2)
c.....shell labels, energies, occupancies and real-space cutoff index
      dimension nnlz(idim2),ee(idim2),wwnl(idim2),nkkk(idim2),
     +  krel(idim2)
c.,...work arrays for the tcm routine call
      dimension bndmat(idim1,5),npivot(idim1)
c.....pseudo quantum numbers, energies occupancies and cutoff radii
      dimension nnlzps(idim4),eeps(idim4),wwnlps(idim4),rc(idim5),
     +rinner(idim6)
c.....beta and q functions and q pseudization coefficients
      dimension beta(idim1,idim3),qfunc(idim1,idim3,idim3),
     +qfcoef(idim8,idim6,idim3,idim3),vloc(idim1),vloc0(idim1)
c.....extra fixed points to avoid negative densities when pseudizing q_ij
      dimension ibfix(idim3),rfix(idim3),qfix(idim3)
c.....work space for solution to inhomogeneous equation and overlaps
      dimension eta(idim1,idim7),alpha(idim3,idim2)
c.....angular momenta, reference energy, qij and dij of vanderbilt scheme
      dimension lll(idim3),eee(idim3),iptype(idim3),qqq(idim3,idim3),
     +ddd0(idim3,idim3),ddd(idim3,idim3),sss(idim3,idim3),
     +ttt(idim3,idim3),bbb(idim3,idim3),bbbi(idim3,idim3),iwork(idim3)
c.....indexing for the beta functions
      dimension nbl0(idim5),nbl(idim5),nbl0d(idim5),nbld(idim5)
c.....psi,phi and chi functions for generation of vanderbilt potential
      dimension psi(idim1,idim3),phi(idim1,idim3),chi(idim1,idim3)
c.....version number and date
      dimension iver(3),idmy(3)
c.....scratch arrays
      dimension xwa(idim1,4)
c
c.....title, and exchange correlation type
      character*20 title,xctype
c.....file name array
      character*40 flname(6)
c
      external idate
c
c=======================================================================
c     c h a n g e s  m a d e  t o  c r e a t e  v e r s i o n  3.0.0
c=======================================================================
c
c     [1] added nodless pseudization of wavefunctions using exponential
c         form.  controlled by iptype switch: 
c                if  iptype = 0    original polynomial pseudization
c                if  iptype = 1    exponential pseudization
c
c     [2] added new pseudization of the local potential to get log
c         derivative correct in one particular angular momentum channel
c         at one particular energy with local potential alone.  
c         controlled by lloc and eloc switches:
c                if  lloc  = -1    original pseudization scheme
c                if  lloc  = +ve   get l'deriv correct in channel lloc
c                at the energy specified by eloc
c
c     [3] added new pseudization of q_ij function using rappe, rabe,
c         kaxiras and joannopoulos type approach.  controlled by the
c         switches ifqopt, nqf and qtryc switches:
c                if  ifqopt = 0    original polynomial pseudization
c                if  ifqopt = 1    new scheme with taylor expansion
c                of nqf terms optimising for cutoff wavevector qtryc
c
c     [4] added version number and date to program.  version number
c         should greatly ease the maintanance of backward compatibilty
c         between potential and car-parrinello program in future 
c         upgrades
c
c     [5] split the variable nvales into two new variables nvales and
c         nang.  nang is now lmax + 1, where lmax is the maximum angular
c         momentum of the beta functions.  nvales is the number of 
c         states to be treated as valence states.  this allows greater
c         flexibilty in the use of the programs and for example 
c         affords the oportunity to treat 3s,3p,4s and 3d states in
c         titanium as valence states using only up to d non-locality
c         in the projector functions.
c
c     [6] general improvement of print outs and fourier transforms of
c         q_ij and the pseudo wavefunctions
c
c     [7] added sweeping of energy on first call to schgef on
c         pseudopotential transferability tests to aid convergence.
c
c=======================================================================
c     c h a n g e s  m a d e  t o  c r e a t e  v e r s i o n  4.0.0
c=======================================================================
c
c     [8] added soft pseudization of the wavefunctions.  controlled by 
c         iptype switch: 
c                if  iptype = 2    soft pseudization
c         cutoff and number of terms in taylor series set by nqf and
c         qtryc parameters.
c
c     [9] added check to ensure that there are no negative densities
c         introduced in the process of pseudizing q_ij function. 
c
c    [10] added source code of nag routines for generalized eigenvalue
c         problems (instead of eispak).
c
c    [11] added printout to check tails of potentials are precisely
c         zv / r
c
c    [12] added routines for relativistic corrections.  controlled
c         by the switch irel.  irel = 1 is for dirac calculations
c         (all electron only).  irel = 2 is for koelling and harmon
c         scalar relativistic equation (all electron and pseudopotential
c         tests and generation).
c
c=======================================================================
c     c h a n g e s  m a d e  t o  c r e a t e  v e r s i o n  4.1.0
c=======================================================================
c
c    [13] added extra constraint to force charge density to be positive
c         when pseudizing q_ij (ifqopt=2).
c
c=======================================================================
c     c h a n g e s  m a d e  t o  c r e a t e  v e r s i o n  5.0.0
c=======================================================================
c
c    [14] when ifqopt=2, program now pseudizes q_ij correctly
c         for each l = l_min,l_max,2 .  also there are cosmetic
c         improvements, eg in printouts.
c
c=======================================================================
c     c h a n g e s  m a d e  t o  c r e a t e  v e r s i o n  5.0.1
c=======================================================================
c
c    [15] treatment of tail of atom is improved, so that the "alpha"
c         term used to correct the ewald energy is less sensitive
c         to numerical errors:
c         (a) on last self-consistent iteration, charge is updated
c             without damping
c         (b) a polynomial interpolation is used to take exc and uxc
c             smoothly to zero for densities less than 1.0d-15.
c         (c) program checks convergence of alpha and stops if need be.
c
c=======================================================================
c     c h a n g e s  m a d e  t o  c r e a t e  v e r s i o n  5.1.0
c=======================================================================
c
c    [16] allows variable r_inner as a function of l for pseudization
c         of the q_ij
c         additional plot of all-electron and L-dependent pseudized
c         q_ij function
c
c=======================================================================
c     c h a n g e s  m a d e  t o  c r e a t e  v e r s i o n  5.1.2
c=======================================================================
c
c    [17] fixed a couple of (hopefully minor) bugs and made the propram
c         more portable to a variety of compilers
c         (a) pssubs.f: change dvs3 to dvs(3).  this error was serious
c               and caused the fitting of phi's for l=3 to be slightly
c               less than optimal (fixed in 5.1.1)
c         (b) dimensions of con in pswf2 were not handled correctly
c               (this bug does not appear to have caused any damage)
c         (c) fix attempt to reference nbl(4) when doing f log derivs
c         (d) miscellaneous other changes and fixes
c
c=======================================================================
c
c
c.....set the version number
      data (iver(i),i=1,3) /5,1,2/
c
c.....variable for file = 0
      integer stderr
      common /files/ stderr,input,iout,ioae,iplot,iologd,iops
c
c     unit stderr=0 automatically connected to stderr
      write (stderr,20) (iver(i),i=1,3)
   20 format('beginning execution pseudopotential program version',i3,
     +  '.',i1,'.',i1)
c
c     set up the files common block
      stderr = 0
      input  = 10 
      iout   = 6
      ioae   = 3
      iplot  = 15
      iologd = 16
      iops   = 14
c     
c     get filenames from command argument string
      do 10 ifile=1,6
        call getarg(ifile,flname(ifile))
   10 continue
c
c     get the date of current run
      call idate(idmy)
c
c     open the file for standard out put of the program
      open( unit = iout , file = flname(2) , status = 'unknown' )
c
c     print a header line in output file
      write(iout,30) (iver(i),i=1,3),idmy(2),idmy(1),idmy(3)
   30 format(72('='),/,'pseudopotential program version',i3,'.',
     +  i1,'.',i1,'   date:',i3,' - ',i2,' - ',i4,/,72('='))
c
      call atom(r,rab,sqr,ruae,runuc,ruhar,reexch,ruexch,
     +  rscore,rsvale,rsatom,rspsco,snl,snlo,nnlz,ee,wwnl,nkkk,
     +  krel,yy,xwa,title,xctype,flname,bndmat,npivot,
     +  nnlzps,eeps,wwnlps,rc,lll,eee,iptype,qqq,ddd0,ddd,
     +  sss,ttt,bbb,bbbi,iwork,alpha,eta,beta,qfunc,qfcoef,
     +  vloc,vloc0,nbl0,nbl,nbl0d,nbld,psi,phi,chi,iver,idmy,
     +  ibfix,rfix,qfix,rinner,
     +  idim1,idim2,idim3,idim4,idim5,idim6,idim7,idim8)
c
      end
c
c----------------------------------------------------------------------------
c
      subroutine atom(r,rab,sqr,ruae,runuc,ruhar,reexch,ruexch,
     +  rscore,rsvale,rsatom,rspsco,snl,snlo,nnlz,ee,wwnl,nkkk,
     +  krel,yy,xwa,title,xctype,flname,bndmat,npivot,
     +  nnlzps,eeps,wwnlps,rc,lll,eee,iptype,qqq,ddd0,ddd,
     +  sss,ttt,bbb,bbbi,iwork,alpha,eta,beta,qfunc,qfcoef,
     +  vloc,vloc0,nbl0,nbl,nbl0d,nbld,psi,phi,chi,iver,idmy,
     +  ibfix,rfix,qfix,rinner,
     +  idim1,idim2,idim3,idim4,idim5,idim6,idim7,idim8)
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
c
c.....logarithmic radial mesh information
      dimension r(idim1),rab(idim1),sqr(idim1)
c.....potentials for the all electron calculation
      dimension ruae(idim1),runuc(idim1)
c.....hartree, exchange and correlation potentials
      dimension ruhar(idim1),reexch(idim1),ruexch(idim1)
c.....charge densities
      dimension rscore(idim1),rsvale(idim1),rsatom(idim1),rspsco(idim1)
c.....wavefunctions
      dimension snl(idim1,idim2),snlo(idim1),yy(idim1,2)
c.....shell labels, energies, occupancies and real-space cutoff index
      dimension nnlz(idim2),ee(idim2),wwnl(idim2),nkkk(idim2),
     +  krel(idim2)
c.,...work arrays for the tcm routine call and relativity routines
      dimension bndmat(idim1,5),npivot(idim1)
c.....pseudo quantum numbers, energies occupancies and cutoff radii
      dimension nnlzps(idim4),eeps(idim4),wwnlps(idim4),rc(idim5)
c.....l-dependent pseudization qfunction cutoff radii
      dimension rinner(idim6)
c.....beta and q functions and q pseudization coefficients
      dimension beta(idim1,idim3),qfunc(idim1,idim3,idim3),
     +qfcoef(idim8,idim6,idim3,idim3),vloc(idim1),vloc0(idim1)
c.....extra fixed points to avoid negative densities when pseudizing q_ij
      dimension ibfix(idim3),rfix(idim3),qfix(idim3)
c.....work space for solution to inhomogeneous equation and overlaps
      dimension eta(idim1,idim7),alpha(idim3,idim2)
c.....angular momenta, reference energy, qij and dij of vanderbilt scheme
      dimension lll(idim3),eee(idim3),iptype(idim3),qqq(idim3,idim3),
     +ddd0(idim3,idim3),ddd(idim3,idim3),sss(idim3,idim3),
     +ttt(idim3,idim3),bbb(idim3,idim3),bbbi(idim3,idim3),iwork(idim3)
c.....indexing for the beta functions
      dimension nbl0(idim5),nbl(idim5),nbl0d(idim5),nbld(idim5)
c.....psi,phi and chi functions for generation of vanderbilt potential
      dimension psi(idim1,idim3),phi(idim1,idim3),chi(idim1,idim3)
c.....version number and date
      dimension iver(3),idmy(3)
c.....logic to determine whether to scan in scheqg
      logical lsweep
c.....scratch arrays
      dimension xwa(idim1,4)
c
c.....title, and exchange correlation type
      character*20 title,xctype
c.....file name array
      character*40 flname(6)
c
      integer stderr
      common /files/ stderr,input,iout,ioae,iplot,iologd,iops
c
c     ------------------------------------------------------
c     note: in this program:
c     potentials, e.g. rucore, are really r*v(r)
c     wave funcs, e.g. snl, are really proportional to r*psi(r)
c       and are normalized so int dr (snl**2) = 1
c     thus psi(r-vec)=(1/r)*snl(r)*y_lm(theta,phi)
c     conventions carry over to beta, etc
c     charge dens, e.g. rscore, really 4*pi*r**2*rho
c     ------------------------------------------------------
c
c     rydberg units are used throughout this program
c
c     ------------------------------------------------------
c
c     ******************************************
c     r o u t i n e  i n i t i a l i s a t i o n
c     ******************************************
c
      call second(tbeg)
 9999 format ('0elapsed time',f10.2,' sec')
c
c     read in the basic parameters determining the run
      call readin( + 1 ,flname,ifae,ifpsp,ifprt,ifplw,ilogd,
     +  rlogd,emin,emax,nnt,thresh,tol,damp,maxit,title,z,xion,
     +  exfact,xctype,rmax,aasf,bbsf,ncspvs,irel,nnlz,wwnl,ee,
     +  ncores,nvales,nang,keyps,lmaxps,ifpcor,rinner,nbeta,rcloc,
     +  rc,lll,eee,iptype,nbl,nbl0,zv,lloc,eloc,ifqopt,nqf,qtryc,
     +  ibfix,rfix,qfix,nfix,idim2,idim3,idim4,idim5,idim6,idim7,
     +  idim8)
c
c     decide where to get the data for the all electron calculation
      if ( ifae .eq. 0 ) then
c
c       read in the data from a previous run
        call rwae( + 1 ,z,xion,exfact,mesh,irel,aasf,bbsf,rmax,
     +    ncspvs,nnlz,ee,wwnl,nkkk,snl,ruae,flname,idim1,idim2)
c
      elseif ( ifae .eq. 1 ) then
c
c       read in controls for a new all electron calculation
        call readin( +2 ,flname,ifae,ifpsp,ifprt,ifplw,ilogd,
     +    rlogd,emin,emax,nnt,thresh,tol,damp,maxit,title,z,xion,
     +    exfact,xctype,rmax,aasf,bbsf,ncspvs,irel,nnlz,wwnl,ee,
     +    ncores,nvales,nang,keyps,lmaxps,ifpcor,rinner,nbeta,rcloc,
     +    rc,lll,eee,iptype,nbl,nbl0,zv,lloc,eloc,ifqopt,nqf,qtryc,
     +    ibfix,rfix,qfix,nfix,idim2,idim3,idim4,idim5,idim6,idim7,
     +    idim8)
c
      endif
c
      if ( ifpsp .gt. 0 ) then
c
c       read in controls for pseudopotential test or generation
        call readin( +3 ,flname,ifae,ifpsp,ifprt,ifplw,ilogd,
     +    rlogd,emin,emax,nnt,thresh,tol,damp,maxit,title,z,xion,
     +    exfact,xctype,rmax,aasf,bbsf,ncspvs,irel,nnlz,wwnl,ee,
     +    ncores,nvales,nang,keyps,lmaxps,ifpcor,rinner,nbeta,rcloc,
     +    rc,lll,eee,iptype,nbl,nbl0,zv,lloc,eloc,ifqopt,nqf,qtryc,
     +    ibfix,rfix,qfix,nfix,idim2,idim3,idim4,idim5,idim6,idim7,
     +    idim8)
c
c
      endif
c
c
c     generate the logarithmic mesh
      call rinit(r,rab,sqr,rmax,aasf,bbsf,a,b,z,rlogd,klogd,
     +  mesh,ifprt,idim1)
c
c     establish the starting nuclear charge configuration
      call setnuc(z,r,runuc,sigma,mesh,idim1,ifprt)
c
c     possible generation of an analytic starting potential
      if ( ifae .eq. 1 ) then
c
        ipass = 1
        call startv(ipass,z,xion,mesh,r,runuc,ruae,
     +    wwnl,wwnlps,nvales,nvalps,ncores,zv,vloc,vloc0,
     +    idim1,idim2,idim4)
c
       endif
c
c     *********************************************
c     a l l  e l e c t r o n  c a l c u l a t i o n
c     *********************************************
c
      write (stderr,*) 'beginning the all electron calculation'
c
      if ( irel .eq. 0 ) then
        write(iout,9998) irel,'schroedinger'
      elseif ( irel .eq. 1 ) then
        write(iout,9998) irel,'dirac'
      elseif ( irel .eq. 2 ) then
        write(iout,9998) irel,'koelling-harmon'
      endif
 9998 format(' irel =',i2,' so all electron calculations',
     +  ' use ',a,' equations')
c
c     timings
      call second(tnow)
      write (stderr,9999) (tnow-tbeg)
      write (iout,9999) (tnow-tbeg)
c
c     iterative loop to converge the potential of all electron calculation
      do 100 iter = 1,maxit
c
c
        if ( irel .eq. 0 ) then
c
c         solve the schroedinger equation
          call schsol(ruae,r,rab,sqr,a,b,xwa(1,1),z,ee,nnlz,wwnl,nkkk,
     +      snl,snlo,ncspvs,mesh,thresh,ifprt,idim1,idim2)
c
        elseif ( irel .eq. 1 ) then
c
c         solve the dirac equation
          call dirsol(ruae,r,rab,bndmat,ee,nnlz,wwnl,nkkk,krel,
     +      snl,yy,ncspvs,mesh,thresh,ifprt,idim1,idim2)
c
        elseif ( irel .eq. 2 ) then
c
          call koesol(ruae,r,rab,bndmat,ee,nnlz,wwnl,nkkk,
     +      snl,yy,xwa,ncspvs,mesh,thresh,ifprt,idim1,idim2)
c
        endif
c
c       compute the charge density from the wave functions
        call rsae(snl,wwnl,nkkk,nnlz,rscore,rsvale,ncores,ncspvs,
     +    mesh,alpha,nbl,nbl0,rab,1,0,kkbeta,qfunc,xwa(1,1),
     +    idim1,idim2,idim3,idim5)
c
c       compute the exchange and correlation potential
        call vhxc( + 1 ,ifpcor,ruexch,ruhar,reexch,rscore,
     +    rsvale,rsatom,rspsco,xwa(1,1),exfact,r,rab,
     +    xwa(1,2),xwa(1,3),xwa(1,4),ehar,emvxc,mesh,idim1)
c
c       test for self-consistency and mix old and new potentials
        call mixer(ruae,runuc,ruhar,ruexch,xwa(1,1),damp,
     +    tol,delta,iselfc,mesh,idim1)
c
c       check whether self-consistency has been achieved
        if ( iselfc .eq. + 1 ) goto 200
c
c       check whether iterations complete but still not consistent
        if ( iselfc .eq. - 1 .and. iter .eq. maxit ) then
          write(iout,*) '***error in subroutine atom'
          write(iout,*) 'potential could not be converged'
          call exit(1)
        endif
c
        write(stderr,9990) iter,delta
 9990   format('completed self-consistent cycle',i4,' delta =',d15.7)
c
  100 continue
c
  200 continue
c
c     compute the total energy of the atom and print results
      call eigprt(delta,nnlz,ee,wwnl,nkkk,ncspvs,ehar,emvxc,a,b,
     +  r,ruae,rsatom,rscore,xwa(1,1),mesh,ifprt,idim1,idim2)
c
c     write the results of the all electron calculation to file
      if ( ifae .eq. 1 ) then
        call rwae( - 1 ,z,xion,exfact,mesh,irel,aasf,bbsf,rmax,
     +    ncspvs,nnlz,ee,wwnl,nkkk,snl,ruae,flname,idim1,idim2)
      endif
c
c     timings
      call second(tnow)
      write (stderr,9999) (tnow-tbeg)
      write (iout,9999) (tnow-tbeg)
c
c     set up for print outs of wavefunctions
      ipass = 1
      call prwf(ifprt,ncores,nvales,ncspvs,nnlz,ee,nkkk,
     +  r,rab,a,b,rsvale,snl,ipass,flname,ifplw,mesh,idim1,idim2)
c
c     compute logarithmic derivatives
      zz = 0.0d0
      vzero = ruae(2) / r(2)
      ipass = 1
c
c     no logderivatives if irel is 1 (dirac equation)
      if ( irel .ne. 1 )
     +  call lderiv(ifprt,ipass,keyps,ilogd,klogd,irel,emax,emin,nnt,
     +  zz,vzero,b,r,rab,sqr,ruae,xwa(1,1),snlo,yy,flname,mesh,
     +  vloc,beta,nbl,nbl0,xwa(1,2),alpha,eta,kkbeta,ddd,qqq,
     +  bndmat,idim1,idim3,idim5,idim7)
c
      write(stderr,*) 'all electron calculation completed'
c
      if ( ifpsp .eq. 0 ) return
c
c     **************************************
c     test of the ultra soft pseudopotential
c     **************************************
c
      if ( ifpsp .eq. 1 ) then
c
        write(stderr,*) 'reading the pseudopotential from file'
c
        call rwps( + 1 ,title,zps,zvps,exftps,nvalps,irel,
     +    z,zv,exfact,nvales,mesh,etot,nnlzps,wwnlps,eeps,
     +    nnlz,wwnl,ee,ncores,keyps,ifpcor,rinner,rc,nbeta,
     +    kkbeta,lll,eee,beta,ddd0,ddd,qqq,qfunc,qfcoef,
     +    rcloc,vloc0,vloc,lloc,eloc,rsatom,rspsco,flname,
     +    ifqopt,nqf,qtryc,nang,nbl0,nbl,r,rab,iver,idmy,
     +    idim1,idim2,idim3,idim4,idim5,idim6,idim8)
c
c       for pseudopotential test ipass parameter is + 2
        ipass = 2
c
c       generate a starting guess for the potential
        call startv(ipass,z,xion,mesh,r,runuc,ruae,
     +    wwnl,wwnlps,nvales,nvalps,ncores,zv,vloc,vloc0,
     +    idim1,idim2,idim4)
c
c       generate ddd for this starting potential
        keydir = + 1
        call dddscr(keydir,r,rab,vloc,qfunc,xwa(1,1),kkbeta,nbeta,
     +    lll,ddd,ddd0,mesh,ifprt,idim1,idim3)
c
        write(stderr,*) 'beginning pseudopotential test'
c
c       iterative loop to converge the test pseudopotential configuration
        do 300 iter = 1,maxit
c
c         scan energy in scheqg if first cycle
          if ( iter .eq. 1 ) then
            lsweep = .true.
          else
            lsweep = .false.
          endif
c
c         solve the schroedinger equation
          call scheqg(vloc,r,rab,sqr,a,b,xwa(1,1),z,ee,rcloc,rc,
     +      nnlz,wwnl,nkkk,snl,snlo,ncores,ncspvs,mesh,
     +      thresh,beta,nbl,nbl0,xwa(1,2),alpha,eta,klogd,kkbeta,
     +      ddd,qqq,ifprt,lsweep,idim1,idim2,idim3,idim5,idim7)
c
c         compute the charge density from the wave functions
          call rsae(snl,wwnl,nkkk,nnlz,rscore,rsvale,ncores,ncspvs,
     +      mesh,alpha,nbl,nbl0,rab,ipass,keyps,kkbeta,qfunc,xwa(1,1),
     +      idim1,idim2,idim3,idim5)
c
c         compute the exchange and correlation potential
          call vhxc(ipass,ifpcor,ruexch,ruhar,reexch,rscore,
     +      rsvale,rsatom,rspsco,xwa(1,1),exfact,r,rab,
     +      xwa(1,2),xwa(1,3),xwa(1,4),ehar,emvxc,mesh,idim1)
c
c         test for self-consistency and mix old and new potentials
          call mixer(vloc,vloc0,ruhar,ruexch,xwa(1,1),damp,
     +      tol,delta,iselfc,mesh,idim1)
c
c         progress report
          write(stderr,9989) iter,delta
 9989     format('completed self-consistent cycle',i4,' delta =',d15.7)
c
c         check whether self-consistency has been achieved
          if ( iselfc .eq. + 1 ) goto 400
c
c         update the ddd matrix
          keydir = + 1
          call dddscr(keydir,r,rab,vloc,qfunc,xwa(1,1),kkbeta,nbeta,
     +      lll,ddd,ddd0,mesh,ifprt,idim1,idim3)
c
c         check whether iterations complete but still not consistent
          if ( iselfc .eq. - 1 .and. iter .eq. maxit ) then
            write(iout,*) '***error in subroutine atom'
            write(iout,*) 'potential could not be converged'
            call exit(1)
          endif
c
  300   continue
c
  400   continue
        write(stderr,*) 'pseudopotential test converged'
c
      endif
c
c     *************************************
c     ultra soft pseudopotential generation
c     *************************************
c
      if ( ifpsp .eq. 2 .and. keyps .eq. 3 ) then
c
c       generate the vanderbilt soft core pseudopotential
        call scpgef(r,rab,sqr,a,b,rlogd,klogd,irel,ruae,
     +    zz,vzero,mesh,ifprt,
     +    lmaxps,nbeta,rcloc,rc,rinner,lll,eee,iptype,nbl,nbl0,nang,
     +    ddd,ddd0,kkbeta,beta,qqq,qfunc,qfcoef,ifqopt,nqf,qtryc,
     +    lloc,eloc,sss,ttt,bbb,bbbi,iwork,ibfix,rfix,qfix,nfix,
     +    psi,phi,chi,vloc,yy,bndmat,npivot,
     +    xwa(1,2),xwa(1,3),xwa(1,4),
     +    idim1,idim3,idim5,idim6,idim7,idim8)
c
c       print the real-space qfunctions
        call prqf(ifprt,qfunc,kkbeta,nbeta,r,a,b,idim1,idim3)
c
        write(stderr,*) 'fourier transforming the qfunctions'
c
c       fourier transform and print reciprocal-space q functions
        call fanalq(r,rab,qfunc,nbeta,xwa(1,1),mesh,ifprt,
     +    idim1,idim3)
c
c       solve the schroedinger equation
c
c       no scanning of log derivatives so lsweep = .false.
        lsweep = .false.
c
        call scheqg(vloc,r,rab,sqr,a,b,xwa(1,1),z,ee,rcloc,rc,
     +    nnlz,wwnl,nkkk,snl,snlo,ncores,ncspvs,mesh,
     +    thresh,beta,nbl,nbl0,xwa(1,2),alpha,eta,klogd,kkbeta,
     +    ddd,qqq,ifprt,lsweep,idim1,idim2,idim3,idim5,idim7)
c
c       correct the charge density
        call rsae(snl,wwnl,nkkk,nnlz,rscore,rsvale,ncores,ncspvs,
     +    mesh,alpha,nbl,nbl0,rab,2,keyps,kkbeta,qfunc,xwa(1,1),
     +    idim1,idim2,idim3,idim5)
c
        write(stderr,*) 'descreening the potential'
c
c       descreen both the vloc and ddd in the vanderbilt scheme
        call descrn(vloc,vloc0,ddd,ddd0,ifpcor,mesh,
     +    ruexch,ruhar,reexch,rscore,rsvale,rsatom,rspsco,
     +    exfact,r,rab,xwa,etot,wwnl,ee,ncores,ncspvs,zv,
     +    keyps,nbeta,kkbeta,lll,qfunc,ifprt,idim1,idim2,idim3)
c
        write(stderr,*) 'writing the pseudopotential to file'
c
        call rwps( - 1 ,title,z,zv,exfact,nvales,irel,
     +    z,zv,exfact,nvales,mesh,etot,
     +    nnlz(ncores+1),wwnl(ncores+1),ee(ncores+1),
     +    nnlz,wwnl,ee,ncores,keyps,ifpcor,rinner,rc,nbeta,
     +    kkbeta,lll,eee,beta,ddd0,ddd,qqq,qfunc,qfcoef,
     +    rcloc,vloc0,vloc,lloc,eloc,rsatom,rspsco,flname,
     +    ifqopt,nqf,qtryc,nang,nbl0,nbl,r,rab,iver,idmy,
     +    idim1,idim2,idim3,idim4,idim5,idim6,idim8)
c
        write(stderr,*) 'computing logarithmic derivatives of vloc'
c
        write(iout,9995)
c
 9995   format(//,' computing logarithmic derivs for vloc',//)
c
        do 500 i = 1,nang
          nbl0d(i) = 0
          nbld(i) = 0
  500   continue
c

c       compute logarithmic derivatives
        zz = 0.0d0
        vzero = ruae(2) / r(2)
        ipass = 2
        call lderiv(ifprt,ipass,keyps,ilogd,klogd,irel,emax,emin,nnt,
     +    zz,vzero,b,r,rab,sqr,ruae,xwa(1,1),snlo,yy,flname,mesh,
     +    vloc,beta,nbld,nbl0d,xwa(1,2),alpha,eta,kkbeta,ddd,qqq,
     +    bndmat,idim1,idim3,idim5,idim7)
c
      endif
c
c     set up for print outs of wavefunctions
      ipass = 2
      call prwf(ifprt,ncores,nvales,ncspvs,nnlz,ee,nkkk,
     +  r,rab,a,b,rsvale,snl,ipass,flname,ifplw,mesh,idim1,idim2)
c
c     fourier analyse the pseudo wavefunctions
      write(stderr,*) 'fourier analysing pseudowavefunctions'
      call fanal(r,rab,nnlz,wwnl,nkkk,snl,ncores,
     +  ncspvs,xwa(1,1),mesh,ifprt,idim1,idim2)
c
c     compute logarithmic derivatives
      write(stderr,*) 'computing the logarithmic derivatives'
      zz = 0.0d0
      vzero = ruae(2) / r(2)
      ipass = 2
      call lderiv(ifprt,ipass,keyps,ilogd,klogd,irel,emax,emin,nnt,
     +  zz,vzero,b,r,rab,sqr,ruae,xwa(1,1),snlo,yy,flname,mesh,
     +  vloc,beta,nbl,nbl0,xwa(1,2),alpha,eta,kkbeta,ddd,qqq,
     +  bndmat,idim1,idim3,idim5,idim7)
c
      return
      end
c
